@@ -22,8 +22,6 @@
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
 #include <sys/time.h>
 #include <time.h>
 #include "error.h"
