@@ -172,6 +172,7 @@ static pthread_t thread;
 static int pipefd[2];
 static pthread_mutex_t mutex_input_ready = PTHREAD_MUTEX_INITIALIZER;
 static char line[PATH_MAX];
+static size_t listing_device_name_length_max = 0;
 
 static void optional_local_echo(char c)
 {
@@ -711,9 +712,12 @@ void handle_command_sequence(char input_char, char *output_char, bool *forward)
                         tio_printf_raw("Enter file name: ");
                         if (tio_readln())
                         {
+                            int ret;
+
                             tio_printf("Sending file '%s'  ", line);
                             tio_printf("Press any key to abort transfer");
-                            tio_printf("%s", xymodem_send(device_fd, line, XMODEM_1K) < 0 ? "Aborted" : "Done");
+                            ret = xymodem_send(device_fd, line, XMODEM_1K);
+                            tio_printf("%s", ret < 0 ? "Aborted" : "Done");
                         }
                         break;
 
@@ -722,9 +726,12 @@ void handle_command_sequence(char input_char, char *output_char, bool *forward)
                         tio_printf_raw("Enter file name: ");
                         if (tio_readln())
                         {
+                            int ret;
+
                             tio_printf("Sending file '%s'  ", line);
                             tio_printf("Press any key to abort transfer");
-                            tio_printf("%s", xymodem_send(device_fd, line, XMODEM_CRC) < 0 ? "Aborted" : "Done");
+                            ret = xymodem_send(device_fd, line, XMODEM_CRC);
+                            tio_printf("%s", ret < 0 ? "Aborted" : "Done");
                         }
                         break;
 
@@ -733,9 +740,12 @@ void handle_command_sequence(char input_char, char *output_char, bool *forward)
                         tio_printf_raw("Enter file name: ");
                         if (tio_readln())
                         {
+                            int ret;
+
                             tio_printf("Ready to receiving file '%s'  ", line);
                             tio_printf("Press any key to abort transfer");
-                            tio_printf("%s", xymodem_receive(device_fd, line, XMODEM_CRC) < 0 ? "Aborted" : "Done");
+                            ret = xymodem_send(device_fd, line, XMODEM_CRC);
+                            tio_printf("%s", ret < 0 ? "Aborted" : "Done");
                         }
                         break;
 
@@ -1093,9 +1103,12 @@ void handle_command_sequence(char input_char, char *output_char, bool *forward)
                 tio_printf("Send file with YMODEM");
                 tio_printf_raw("Enter file name: ");
                 if (tio_readln()) {
+		    int ret;
+
                     tio_printf("Sending file '%s'  ", line);
                     tio_printf("Press any key to abort transfer");
-                    tio_printf("%s", xymodem_send(device_fd, line, YMODEM) < 0 ? "Aborted" : "Done");
+		    ret = xymodem_send(device_fd, line, YMODEM);
+                    tio_printf("%s", ret < 0 ? "Aborted" : "Done");
                 }
                 break;
 
@@ -1407,6 +1420,14 @@ static bool is_serial_device(const char *format, ...)
         return false;
     }
 
+#if defined(__APPLE__)
+    // Make sure device name is on the form /dev/cu.* or /dev/tty.*
+    if ((strncmp(filename, "/dev/cu.", 8) != 0) && (strncmp(filename, "/dev/tty.", 9) != 0))
+    {
+        return false;
+    }
+#endif
+
     fd = open(filename, O_RDONLY | O_NONBLOCK | O_NOCTTY);
     if (fd == -1)
     {
@@ -1619,6 +1640,9 @@ static void search_reset(void)
 
     // Indicate an empty list
     device_list = NULL;
+
+    // Reset max device name length
+    listing_device_name_length_max = 0;
 }
 
 #if defined(__linux__)
@@ -1785,6 +1809,12 @@ GList *tty_search_for_serial_devices(void)
 
         // Add device information to device list
         device_list = g_list_append(device_list, device);
+
+        // Update length of longest device name string
+        if (strlen(device->path) > listing_device_name_length_max)
+        {
+            listing_device_name_length_max = strlen(device->path);
+        }
     }
 
     if (g_list_length(device_list) == 0)
@@ -1858,13 +1888,19 @@ GList *tty_search_for_serial_devices(void)
 
         // Fill in device information
         device->path = g_strdup(path);
-        device->tid = "";
+        device->tid = g_strdup("");
         device->uptime = uptime;
-        device->driver = "";
-        device->description = "";
+        device->driver = g_strdup("");
+        device->description = g_strdup("");
 
         // Add device information to device list
         device_list = g_list_append(device_list, device);
+
+        // Update length of longest device name string
+        if (strlen(device->path) > listing_device_name_length_max)
+        {
+            listing_device_name_length_max = strlen(device->path);
+        }
     }
 
     if (g_list_length(device_list) == 0)
@@ -1889,8 +1925,14 @@ void list_serial_devices(void)
 
     if (g_list_length(device_list) > 0)
     {
-        printf("Device            TID     Uptime [s] Driver           Description\n");
-        printf("----------------- ---- ------------- ---------------- --------------------------\n");
+        if (listing_device_name_length_max < 17)
+        {
+            listing_device_name_length_max = 17;
+        }
+        print_padded("Device", listing_device_name_length_max, ' ');
+        printf(" TID     Uptime [s] Driver           Description\n");
+        print_padded("", listing_device_name_length_max, '-');
+        printf(" ---- ------------- ---------------- --------------------------\n");
 
         // Iterate through the device list
         GList *iter;
@@ -1899,7 +1941,8 @@ void list_serial_devices(void)
             device_t *device = (device_t *) iter->data;
 
             // Print device information
-            printf("%-17s %4s %13.3f %-16s %s\n", device->path, device->tid, device->uptime, device->driver, device->description);
+            print_padded(device->path, listing_device_name_length_max, ' ');
+            printf(" %4s %13.3f %-16s %s\n", device->tid, device->uptime, device->driver, device->description);
         }
         printf("\n");
     }
@@ -2082,7 +2125,7 @@ void tty_wait_for_device(void)
         }
         else if (last_errno != errno)
         {
-            tio_warning_printf("Could not open tty device (%s)", strerror(errno));
+            tio_warning_printf("Could not open %s (%s)", device_name, strerror(errno));
             tio_printf("Waiting for tty device..");
             last_errno = errno;
         }
@@ -2201,11 +2244,17 @@ void forward_to_tty(int fd, char output_char)
                 {
                     handle_hex_prompt(output_char);
                 }
-                else
+                else if (option.input_mode == INPUT_MODE_NORMAL)
                 {
-                    if (option.input_mode == INPUT_MODE_NORMAL)
+                    status = tty_write(device_fd, &output_char, 1);
+                    if (status < 0)
+                    {
+                        tio_warning_printf("Could not write to tty device");
+                    }
+                    else
                     {
                         optional_local_echo(output_char);
+                        tx_total++;
                     }
                 }
                 break;
@@ -2612,22 +2661,22 @@ int tty_connect(void)
                                     break;
 
                                 case INPUT_MODE_LINE:
-                                    switch (input_char)
+                                    if (input_char == '\r')
                                     {
-                                        case '\r': // Carriage return
-                                            readline_input(input_char);
+                                        // Carriage return
+                                        readline_input(input_char);
 
-                                            // Write current line to tty device
-                                            char *line = readline_get();
-                                            tty_write(device_fd, line, strlen(line));
-                                            tty_sync(device_fd);
-                                            break;
-
-                                        default:
-                                            readline_input(input_char);
-                                            forward = false;
-                                            break;
+                                        // Write current line to tty device
+                                        char *rl_line = readline_get();
+                                        tty_write(device_fd, rl_line, strlen(rl_line));
+                                        tty_sync(device_fd);
                                     }
+                                    else
+                                    {
+                                        readline_input(input_char);
+                                        forward = false;
+                                    }
+                                    break;
 
                                 default:
                                     break;
